﻿/**
 *	@brief		Declares the TPM_TakeOwnership command.
 *	@details	The module receives the input parameters, marshals these parameters
 *				to a byte array, sends the command to the TPM and unmarshals the response
 *				back to the out parameters.
 *	@file		TPM_TakeOwnership.h
 *	@copyright	Copyright 2014 - 2017 Infineon Technologies AG ( www.infineon.com )
 *
 *	@copyright	All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 *  1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the copyright holder nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#pragma once

#include "TPM_Types.h"

#ifdef __cplusplus
extern "C" {
#endif

/**
 *	@brief		This function handles the TPM_TakeOwnership command
 *	@details	The function receives the input parameters marshals these parameters
 *				to a byte array sends the command to the TPM and unmarshals the response
 *				back to the out parameters
 *
 *	@param		PrgbEncryptedOwnerAuth		The owner AuthData encrypted with PUBEK
 *	@param		PunEncryptedOwnerAuthSize	The size of the PrgbEncryptedOwnerAuth field
 *	@param		PrgbEncryptedSrkAuth		The SRK AuthData encrypted with PUBEK
 *	@param		PunEncryptedSrkAuthSize		The size of the PrgbEncryptedSrkAuth field
 *	@param		PpSrkParams					Structure containing all parameters of new SRK. pubKey.keyLength & encSize are both 0. This structure MAY be TPM_KEY12.
 *	@param		PunAuthHandle				The authorization session handle used for this command
 *	@param		PpOwnerAuth					The owner AuthData
 *	@param		PpNonceEven					On input last even nonce generated by TPM, on output even nonce newly generated by TPM to cover outputs.
 *	@param		PpSrkPub					Structure containing all parameters of new SRK. srkPub.encData is set to 0. This structure MAY be TPM_KEY12.
 *
 *	@retval		RC_SUCCESS				The operation completed successfully.
 *	@retval		RC_E_BAD_PARAMETER		An invalid parameter was passed to the function.
 *	@retval		RC_E_BUFFER_TOO_SMALL	In case an internal buffer is too small for HASH calculation
 *	@retval		RC_E_INTERNAL			In case an error occurred while preparing the HASH buffer
 *	@retval		...						Error codes from called functions.
 */
_Check_return_
unsigned int
TSS_TPM_TakeOwnership(
	_In_bytecount_(PunEncryptedOwnerAuthSize)	BYTE*			PrgbEncryptedOwnerAuth,
	_In_										UINT32			PunEncryptedOwnerAuthSize,
	_In_bytecount_(PunEncryptedOwnerAuthSize)	BYTE*			PrgbEncryptedSrkAuth,
	_In_										UINT32			PunEncryptedSrkAuthSize,
	_In_										TPM_KEY*		PpSrkParams,
	_In_										TPM_AUTHHANDLE	PunAuthHandle,
	_In_										TPM_AUTHDATA*	PpOwnerAuth,
	_Inout_										TPM_NONCE*		PpNonceEven,
	_Out_										TPM_KEY*		PpSrkPub);

#ifdef __cplusplus
}
#endif